@@ -41,7 +41,7 @@ use strict;
 
     require_version DBI 1.61;
 
-    $VERSION = '8.4.1.0001';
+    $VERSION = '8.4.4.0001';
 
     bootstrap DBD::cubrid $VERSION;
 
@@ -125,11 +125,11 @@ use strict;
             $dbname = '';
         }
 
-        my $connect_dsn = "cci:CUBRID:$host:$port:$dbname";
+        my $connect_dsn = "cci:cubrid:$host:$port:$dbname" . ':::';
         my $is_connect_attr = 0;
 
         if ($connect_attr{ALTHOSTS}) {
-            $connect_dsn .= ":::?alhosts=$connect_attr{ALTHOSTS}";
+            $connect_dsn .= "?alhosts=$connect_attr{ALTHOSTS}";
             $is_connect_attr = 1;
         }
 
@@ -137,7 +137,7 @@ use strict;
             if ($is_connect_attr) {
                 $connect_dsn .= "&rctime=$connect_attr{RCTIME}";
             } else {
-                $connect_dsn .= ":::?rctime=$connect_attr{RCTIME}";
+                $connect_dsn .= "?rctime=$connect_attr{RCTIME}";
                 $is_connect_attr = 1;
             }
         }
@@ -146,7 +146,7 @@ use strict;
            if ($is_connect_attr) {
                $connect_dsn .= "&autocommit=$connect_attr{AUTOCOMMIT}";
            } else {
-               $connect_dsn .= ":::?autocommit=$connect_attr{AUTOCOMMIT}";
+               $connect_dsn .= "?autocommit=$connect_attr{AUTOCOMMIT}";
                $is_connect_attr = 1;
            }
         }
@@ -155,7 +155,7 @@ use strict;
             if ($is_connect_attr) {
                 $connect_dsn .= "&login_timeout=$connect_attr{LOGIN_TIMEOUT}";
             } else {
-                $connect_dsn .= ":::?login_timeout=$connect_attr{LOGIN_TIMEOUT}";
+                $connect_dsn .= "?login_timeout=$connect_attr{LOGIN_TIMEOUT}";
                 $is_connect_attr = 1;
             }
         }
@@ -164,7 +164,7 @@ use strict;
             if ($is_connect_attr) {
                 $connect_dsn .= "&query_timeout=$connect_attr{QUERY_TIMEOUT}";
             } else {
-                $connect_dsn .= ":::?query_timeout=$connect_attr{QUERY_TIMEOUT}";
+                $connect_dsn .= "?query_timeout=$connect_attr{QUERY_TIMEOUT}";
                 $is_connect_attr = 1;
             }
         }
@@ -173,7 +173,7 @@ use strict;
             if ($is_connect_attr) {
                 $connect_dsn .= "&disconnect_on_query_timeout=$connect_attr{DISCONNECT_ON_QUERY_TIMEOUT}";
             } else {
-                $connect_dsn .= ":::?disconnect_on_query_timeout=$connect_attr{DISCONNECT_ON_QUERY_TIMEOUT}";
+                $connect_dsn .= "?disconnect_on_query_timeout=$connect_attr{DISCONNECT_ON_QUERY_TIMEOUT}";
                 $is_connect_attr = 1;
             }
         }
@@ -326,6 +326,9 @@ use strict;
         for my $row (@$desc) {
 
             my $type = $row->{type};
+            if (!defined($type)) {
+                $type = "NULL";
+            }
 
             my $info = {
                 TABLE_CAT               => $catalog,
@@ -460,6 +463,9 @@ use strict;
                     $info->{COLUMN_SIZE} = 1073741823;
                 }
             }
+            elsif ($type =~ /^NULL$/) {
+                $info->{DATA_TYPE} = SQL_UNKNOWN_TYPE;
+            }
             else {
                 $info->{DATA_TYPE} = SQL_VARCHAR;
             }
