@@ -35,6 +35,13 @@
 #define  open(file, flag, mode) PerlLIO_open3(file, flag, mode)
 #endif
 
+/* Loading dynamic cascci library needs this header. */
+#ifdef WIN32
+#include <windows.h>
+#else
+#include <dlfcn.h>
+#endif
+
 #define CUBRID_ER_MSG_LEN 1024
 #define CUBRID_BUFFER_LEN 4096
 
@@ -362,20 +369,22 @@ dbd_db_STORE_attrib( SV *dbh, imp_dbh_t *imp_dbh, SV *keysv, SV *valuesv )
 
     switch (kl) {
     case 10:
-        if (strEQ("AutoCommit", key)) {
-            if (on) {
+        if (strEQ("AutoCommit", key)) 
+        {
+            if (on)
+            {
                 cci_set_autocommit (imp_dbh->handle, CCI_AUTOCOMMIT_TRUE);
             }
-            else {
+            else 
+            {
                 cci_set_autocommit (imp_dbh->handle, CCI_AUTOCOMMIT_FALSE);
             }
 
             DBIc_set (imp_dbh, DBIcf_AutoCommit, on);
+            return TRUE;
         }
-        break;
     }
-
-    return TRUE;
+    return FALSE;
 }
 
 /***************************************************************************
@@ -437,7 +446,7 @@ dbd_db_last_insert_id( SV *dbh, imp_dbh_t *imp_dbh,
     int res;
     T_CCI_ERROR error;
 
-    if ((res = cci_last_insert_id (imp_dbh->handle, &name, &error))) {
+    if ((res = cci_get_last_insert_id (imp_dbh->handle, &name, &error))) {
         handle_error (dbh, res, &error);
         return Nullsv;
     }
@@ -446,7 +455,6 @@ dbd_db_last_insert_id( SV *dbh, imp_dbh_t *imp_dbh,
         return Nullsv;
     } else {
         sv = newSVpvn (name, strlen(name));
-        free (name);
     }
 
     return sv_2mortal (sv);
@@ -470,7 +478,7 @@ dbd_db_ping( SV *dbh )
 {
     int res;
     T_CCI_ERROR error;
-    char *query = "SELECT 1+1 from db_root";
+    char *query = "SELECT 1+1 from db_root /*+ shard_id(0) */";
     int req_handle = 0, result = 0, ind = 0;
 
     D_imp_dbh (dbh);
@@ -934,10 +942,11 @@ int
 dbd_bind_ph( SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
              IV sql_type, SV *attribs, int is_inout, IV maxlen )
 {
-    int res;
+    int res = 0;
     char *bind_value = NULL;
     STRLEN bind_value_len;
     T_CCI_ERROR error;
+    int buffer_is_null= 0;
 
     int index = SvIV(param);
     if (index < 1 || index > DBIc_NUM_PARAMS(imp_sth)) {
@@ -947,6 +956,8 @@ dbd_bind_ph( SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
 
     bind_value = SvPV (value, bind_value_len);
 
+    buffer_is_null = !(SvOK(newSVsv(value)) && newSVsv(value));
+
     if (sql_type == SQL_BLOB || sql_type == SQL_CLOB) {
 
         if ((res = _cubrid_lob_bind (sth, 
@@ -961,10 +972,13 @@ dbd_bind_ph( SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
         return TRUE;
     }
 
+    if (! buffer_is_null) {
+        res = cci_bind_param (imp_sth->handle, index, CCI_A_TYPE_STR, bind_value, CCI_U_TYPE_CHAR, 0);
+    } else {
+        res = cci_bind_param (imp_sth->handle, index, CCI_A_TYPE_STR, NULL, CCI_U_TYPE_CHAR, 0);
+    }
 
-    if ((res = cci_bind_param (imp_sth->handle, 
-                    index, CCI_A_TYPE_STR,
-                    bind_value, CCI_U_TYPE_CHAR, 0)) < 0) {
+    if (res < 0) {
         handle_error(sth, res, NULL);
         return FALSE;
     }
@@ -985,6 +999,7 @@ cubrid_st_lob_get( SV *sth, int col )
     T_CCI_U_TYPE u_type;
 
     D_imp_sth (sth);
+    imp_sth->lob = NULL;
 
     if (col < 1 || col > DBIc_NUM_FIELDS (imp_sth)) {
         handle_error (sth, CCI_ER_COLUMN_INDEX, NULL);
@@ -1015,6 +1030,8 @@ cubrid_st_lob_get( SV *sth, int col )
             imp_sth->affected_rows * sizeof (T_CUBRID_LOB)
             );
 
+    memset(imp_sth->lob, 0, imp_sth->affected_rows * sizeof (T_CUBRID_LOB));
+
     while (1) {
 
         if ((res = cci_fetch(imp_sth->handle, &error)) < 0) {
@@ -1070,7 +1087,7 @@ cubrid_st_lob_export( SV *sth, int index, char *filename )
 
     D_imp_sth (sth);
 
-    if (imp_sth->lob[index-1].lob == NULL) {
+    if ( imp_sth->lob == NULL || imp_sth->lob[index-1].lob == NULL) {
         handle_error (sth, CCI_ER_INVALID_LOB_HANDLE, NULL);
         return FALSE;
     }
@@ -1147,6 +1164,13 @@ cubrid_st_lob_import( SV *sth,
         return FALSE;
     }
 
+    if ((fd = open (filename, O_RDONLY, 0400)) < 0) {
+        res = CCI_ER_FILE;
+        handle_error(sth, res, NULL);
+
+        return FALSE;
+    }
+
     if ((res = _cubrid_lob_new (imp_sth->conn, 
                                 &lob,
                                 u_type, 
@@ -1155,11 +1179,6 @@ cubrid_st_lob_import( SV *sth,
         return FALSE;
     }
 
-    if ((fd = open (filename, O_RDONLY, 0400)) < 0) {
-        res = CCI_ER_FILE;
-        goto ER_LOB_IMPORT;
-    }
-
     while (1) {
         if ((size = read (fd, buf, CUBRID_BUFFER_LEN)) < 0) {
             res = CUBRID_ER_READ_FILE;
